package cs108;

import static java.util.Collections.unmodifiableMap;
import static java.util.Collections.unmodifiableSet;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public final class LSystem {
    private final String string;
    private final Map<Character, String> rules;
    private final Set<Character> lineChars;
    private final int turningAngle;

    private LSystem(String string, Map<Character, String> rules, Set<Character> lineChars, int turningAngle) {
        this.string = string;
        this.rules = rules;
        this.lineChars = lineChars;
        this.turningAngle = turningAngle;
    }

    private static Set<Character> setFromString(String s) {
        Set<Character> set = new HashSet<>();
        for (int i = 0; i < s.length(); ++i)
            set.add(s.charAt(i));
        return unmodifiableSet(set);
    }

    public LSystem(String string, Map<Character, String> rules, String lineChars, int turningAngle) {
        this(string, unmodifiableMap(new HashMap<>(rules)), setFromString(lineChars), turningAngle);
    }

    public String string() {
        return string;
    }

    public Map<Character, String> rules() {
        return rules;
    }

    public Set<Character> lineChars() {
        return lineChars;
    }

    public int turningAngle() {
        return turningAngle;
    }

    public LSystem evolve() {
        StringBuilder b = new StringBuilder();
        for (int i = 0; i < string.length(); ++i) {
            char c = string.charAt(i);
            b.append(rules.getOrDefault(c, String.valueOf(c)));
        }
        return new LSystem(b.toString(), rules, lineChars, turningAngle);
    }

    public LSystem evolve(int steps) {
        if (! (steps >= 0))
            throw new IllegalArgumentException();

        LSystem l = this;
        for (int i = 0; i < steps; ++i)
            l = l.evolve();
        return l;
    }
}
