package cs108;

import static javafx.beans.binding.Bindings.divide;
import static javafx.beans.binding.Bindings.max;
import static javafx.beans.binding.Bindings.min;

import java.util.Map;

import javafx.application.Application;
import javafx.beans.binding.NumberBinding;
import javafx.geometry.Bounds;
import javafx.scene.Scene;
import javafx.scene.layout.BorderPane;
import javafx.scene.shape.Path;
import javafx.stage.Stage;

public final class Main extends Application {
    private static LSystem koch() {
        return new LSystem("F", Map.of('F', "F-F++F-F"), "F", 60);
    }

    private static LSystem kochFlake() {
        return new LSystem("F++F++F", Map.of('F', "F-F++F-F"), "F", 60);
    }

    private static LSystem sierpinski() {
        Map<Character, String> rules = Map.of(
                'A', "+B-A-B+",
                'B', "-A+B+A-");
        return new LSystem("A", rules, "AB", 60);
    }

    private static LSystem hilbert() {
        Map<Character, String> rules = Map.of(
                'A', "-BF+AFA+FB-",
                'B', "+AF-BFB-FA+");
        return new LSystem("A", rules, "F", 90);
    }

    private static LSystem tree() {
        return new LSystem("----F", Map.of('F', "FF+[+F-F-F]-[-F+F+F]"), "F", 25);
    }

    private static LSystem plant() {
        Map<Character, String> rules = Map.of(
                'X', "F-[[X]+X]+F[+FX]-X",
                'F', "FF");
        return new LSystem("---X", rules, "F", 30);
    }

    private static LSystem dragon() {
        Map<Character, String> rules = Map.of(
                'X', "X+YF+",
                'Y', "-FX-Y");
        return new LSystem("FX", rules, "F", 90);
    }

    public static void main(String[] args) {
        Application.launch(args);
    }

    @Override
    public void start(Stage primaryStage) throws Exception {
        // Le L-système à dessiner
        LSystem lSystem = tree().evolve(4);

        Path p = LSystemTracer.paint(lSystem);
        BorderPane root = new BorderPane(p);

        Bounds pBounds = p.getBoundsInLocal();
        NumberBinding scale = min(
                divide(root.widthProperty(), pBounds.getWidth()),
                divide(root.heightProperty(), pBounds.getHeight()));
        p.scaleXProperty().bind(scale);
        p.scaleYProperty().bind(scale);
        p.strokeWidthProperty().bind(divide(1, max(1, scale)));

        Scene scene = new Scene(root);
        primaryStage.setMinWidth(200);
        primaryStage.setMinHeight(200);

        primaryStage.setScene(scene);
        primaryStage.setTitle("L-systèmes");
        primaryStage.show();
    }
}
